"""Rio_cogeo.scripts.cli."""

import os
import click

from rasterio.rio import options
from rasterio.enums import Resampling

from rio_cogeo.cogeo import cog_translate
from rio_cogeo.profiles import cog_profiles


class CustomType:
    """Click CustomType."""

    class BdxParamType(click.ParamType):
        """Band inddex type."""

        name = "bidx"

        def convert(self, value, param, ctx):
            """Validate and parse band index."""
            try:
                bands = [int(x) for x in value.split(",")]
                assert all(b > 0 for b in bands)
                return bands

            except (ValueError, AttributeError, AssertionError):
                raise click.ClickException(
                    "bidx must be a string of comma-separated integers (> 0), "
                    "representing the band indexes."
                )

    bidx = BdxParamType()


@click.command()
@options.file_in_arg
@options.file_out_arg
@click.option("--bidx", "-b", type=CustomType.bidx, help="Band index to copy")
@click.option(
    "--cog-profile",
    "-p",
    "cogeo_profile",
    type=click.Choice(cog_profiles.keys()),
    default="ycbcr",
    help="CloudOptimized GeoTIFF profile (default: ycbcr)",
)
@click.option(
    "--nodata", type=int, help="Force mask creation from a given nodata value"
)
@click.option(
    "--alpha", type=int, help="Force mask creation from a given alpha band number"
)
@click.option(
    "--overview-level", type=int, default=6, help="Overview level (default: 6)"
)
@click.option(
    "--overview-resampling",
    help="Resampling algorithm.",
    type=click.Choice([it.name for it in Resampling if it.value in [0, 2, 5, 6, 7]]),
    default="nearest",
)
@click.option("--threads", type=int, default=8)
@options.creation_options
def cogeo(
    input,
    output,
    bidx,
    cogeo_profile,
    nodata,
    alpha,
    overview_level,
    overview_resampling,
    threads,
    creation_options,
):
    """Create Cloud Optimized Geotiff."""
    if nodata is not None and alpha is not None:
        raise click.ClickException('Incompatible options "alpha" and "nodata"')

    output_profile = cog_profiles.get(cogeo_profile)
    output_profile.update(dict(BIGTIFF=os.environ.get("BIGTIFF", "IF_SAFER")))
    if creation_options:
        output_profile.update(creation_options)

    block_size = min(output_profile["blockxsize"], output_profile["blockysize"])

    config = dict(
        NUM_THREADS=threads,
        GDAL_TIFF_INTERNAL_MASK=os.environ.get("GDAL_TIFF_INTERNAL_MASK", True),
        GDAL_TIFF_OVR_BLOCKSIZE=os.environ.get("GDAL_TIFF_OVR_BLOCKSIZE", block_size),
    )

    cog_translate(
        input,
        output,
        output_profile,
        bidx,
        nodata,
        alpha,
        overview_level,
        overview_resampling,
        config,
    )



"""rio_cogeo.cogeo: translate a file to a cloud optimized geotiff."""

import sys

import click

import numpy

import rasterio
from rasterio.io import MemoryFile
from rasterio.enums import Resampling
from rasterio.shutil import copy


def cog_translate(
    src_path,
    dst_path,
    dst_kwargs,
    indexes=None,
    nodata=None,
    alpha=None,
    overview_level=6,
    overview_resampling="nearest",
    config=None,
):
    """
    Create Cloud Optimized Geotiff.

    Parameters
    ----------
    src_path : str or PathLike object
        A dataset path or URL. Will be opened in "r" mode.
    dst_path : str or Path-like object
        An output dataset path or or PathLike object.
        Will be opened in "w" mode.
    dst_kwargs: dict
        output dataset creation options.
    indexes : tuple, int, optional
        Raster band indexes to copy.
    nodata, int, optional
        nodata value for mask creation.
    alpha, int, optional
        alpha band index for mask creation.
    overview_level : int, optional (default: 6)
        COGEO overview (decimation) level
    config : dict
        Rasterio Env options.

    """
    config = config or {}

    with rasterio.Env(**config):
        with rasterio.open(src_path) as src:

            indexes = indexes if indexes else src.indexes
            meta = src.meta
            meta["count"] = len(indexes)
            meta.pop("nodata", None)
            meta.pop("alpha", None)

            meta.update(**dst_kwargs)
            meta.pop("compress", None)
            meta.pop("photometric", None)

            with MemoryFile() as memfile:
                with memfile.open(**meta) as mem:
                    wind = list(mem.block_windows(1))
                    with click.progressbar(
                        wind, length=len(wind), file=sys.stderr, show_percent=True
                    ) as windows:
                        for ij, w in windows:
                            matrix = src.read(window=w, indexes=indexes)
                            mem.write(matrix, window=w)

                            if nodata is not None:
                                mask_value = (
                                    numpy.all(matrix != nodata, axis=0).astype(
                                        numpy.uint8
                                    )
                                    * 255
                                )
                            elif alpha is not None:
                                mask_value = src.read(alpha, window=w)
                            else:
                                mask_value = src.dataset_mask(window=w)
                            mem.write_mask(mask_value, window=w)

                    overviews = [2 ** j for j in range(1, overview_level + 1)]

                    mem.build_overviews(overviews, Resampling[overview_resampling])
                    mem.update_tags(
                        ns="rio_overview",
                        resampling=Resampling[overview_resampling].value,
                    )

                    copy(mem, dst_path, copy_src_overviews=True, **dst_kwargs)



"""Setup."""
from setuptools import setup, find_packages

with open("rio_cogeo/__init__.py") as f:
    for line in f:
        if line.find("__version__") >= 0:
            version = line.split("=")[1].strip()
            version = version.strip('"')
            version = version.strip("'")
            continue


with open("README.rst") as f:
    readme = f.read()

# Runtime requirements.
inst_reqs = ["rasterio[s3]~=1.0"]

extra_reqs = {
    "test": ["pytest", "pytest-cov"],
    "dev": ["pytest", "pytest-cov", "pre-commit"],
}


setup(
    name="rio-cogeo",
    version=version,
    description=u"CloudOptimized GeoTIFF (COGEO) creation plugin for rasterio",
    long_description=readme,
    classifiers=[
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Scientific/Engineering :: GIS",
    ],
    keywords="COGEO CloudOptimized Geotiff rasterio",
    author=u"Vincent Sarago",
    author_email="vincent@mapbox.com",
    url="https://github.com/mapbox/rio-cogeo",
    license="BSD-3",
    packages=find_packages(exclude=["ez_setup", "examples", "tests"]),
    install_requires=inst_reqs,
    extras_require=extra_reqs,
    entry_points="""
      [rasterio.rio_plugins]
      cogeo=rio_cogeo.scripts.cli:cogeo
      """,
)
